/* Aftershock 3D rendering engine
 * Copyright (C) 1999 Stephen C. Taylor
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
#include <stdio.h>
#include <string.h>

#ifdef MACOSX
#include <GLUT/glut.h>
#else
#include <GL/glut.h>
#endif

#include "uicommon.h"
#include "rendercontext.h"
#include "renderhud.h"
#include "globalshared.h"

global_shared_t *g;
static r_context_t *r_context;
static double starttime;

/* use regular memory */
void *gc_malloc(size_t size) { return malloc(size); }
void *rc_malloc(size_t size) { return malloc(size); }
void gc_free(void *mem) { free(mem); }
void rc_free(void *mem) { free(mem); }

static void display(void);
static void mouse(int button, int state, int x, int y);
static void key_down(unsigned char key, int x, int y);
static void key_up(unsigned char key, int x, int y);
static void reshape(int w, int h);

int
main(int argc, char **argv)
{
  /* setup global data structures. */
  g = (global_shared_t *) gc_malloc(sizeof(global_shared_t));
  init_global_shared(g);
  ui_read_args(argc, argv);
  strcpy(g->r_help_fname, "paul/helpGL.jpg");

  /* load map */
  ui_init_bsp();
  printf("\n\n");

  glutInit(&argc, argv);
  glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
  glutInitWindowSize(g->r_viewport_w, g->r_viewport_h);
  glutInitWindowPosition(50, 50);
  glutCreateWindow("");
#if 0 
  glutGameModeString("800x600:16");
  glutEnterGameMode();
#endif

  /* setup rendering context. */
  r_context = (r_context_t *) rc_malloc(sizeof(r_context_t));
  ui_init_gl(r_context);

  if(g->r_fullscreen)
  {
    glutFullScreen();
    glutSetCursor(GLUT_CURSOR_NONE);
  }

  glutDisplayFunc(display);
  glutMouseFunc(mouse);
  glutMotionFunc(ui_mouse_motion);
  glutKeyboardFunc(key_down);
  glutKeyboardUpFunc(key_up);
  glutIdleFunc(display);
  glutReshapeFunc(reshape);

  starttime = (double)glutGet(GLUT_ELAPSED_TIME) / 1000.0;
    
  glutMainLoop();

  ui_finish(r_context);
  return 0;
}

static void
display(void)
{
  double time;

  time = (double)glutGet(GLUT_ELAPSED_TIME) / 1000.0 - starttime;
  ui_move(time);
  ui_display(r_context, time);
  hud_render(r_context);
  glutSwapBuffers();
}

static void
mouse(int button, int state, int x, int y)
{
  ui_mouse_down(x, y);
}

static void
key_down(unsigned char key, int x, int y)
{
  ui_key_down(key);
  if (key == 'r')
    starttime = (double)glutGet(GLUT_ELAPSED_TIME) / 1000.0;
}

static void
key_up(unsigned char key, int x, int y)
{
  ui_key_up(key);
}

static void reshape(int w, int h) {
  g->r_viewport_w = w;
  g->r_viewport_h = h;
  glViewport(0, 0, g->r_viewport_w, g->r_viewport_h);
}
